@@ -47,7 +47,7 @@ def create_excitator(player, label, scaling, convention):
 
     Returns
     -------
-    :py:class:`Exciator<Excitator>`
+    :py:class:`Excitator<Excitator>`
         The rigth excitator object.
 
     """
