@@ -28,18 +28,20 @@ its physical equation.
 import numpy as np
 import warnings
 from openwind.continuous import (Pipe, PhysicalRadiation,
-                                 Scaling, Netlist, JunctionTjoint,
+                                 Scaling, Netlist, JunctionTjoint, JunctionSwitch,
                                  JunctionSimple, losses_model, radiation_model,
                                  JunctionDiscontinuity, create_excitator)
 from openwind.design import ShapeSlice
+from openwind.technical.instrument_geometry import Hole, BrassValve
 
 
 class InstrumentPhysics:
 
     """Create the continuous instrument graph with physical considerations.
 
-    Create the instrument's graph: an ensemble of pipes (main bore and chimney
-    hole) the ends of which being connected to different type of connector:
+    Create the instrument's graph: an ensemble of pipes (main bore, brass valve
+    deviation pipe and chimney hole) the ends of which being connected to
+    different type of connector:
 
     * radiation (:py:class:`PhysicalRadiation\
     <openwind.continuous.physical_radiation.PhysicalRadiation>`)
@@ -67,6 +69,8 @@ class InstrumentPhysics:
             to describe the main bore
         - a list of :py:class:`Hole <openwind.technical.instrument_geometry.Hole>` \
             to describe the holes
+        - a list of :py:class:`BrassValve <openwind.technical.instrument_geometry.BrassValvz>` \
+            to describe the valves
         - a :py:class:`FingeringChart <openwind.technical.fingering_chart.FingeringChart>` \
             object
     temperature : float or callable
@@ -76,7 +80,9 @@ class InstrumentPhysics:
 
             if the temperature is non constant, it is supposed variable
             along the main tube (main bore). In the holes, the temperature is
-            supposed homogeneous.
+            supposed homogeneous. In the valve deviation pipe, the gradient is
+            set to have the right temperature at its two connections with the
+            main bore.
 
     player : :py:class:`Player <openwind.technical.player.Player>`
         the controler for the instrument
@@ -303,19 +309,22 @@ class InstrumentPhysics:
     def __build_netlist(self):
         """
         Build the graph of the instrument by assuming a main bore and
-        eventually some side holes. Each hole chimney and the main bore shapes
-        are converted into pipes connected by junctions and having radiation
-        condition.
+        eventually some side holes and valves deviation pipes. Each deviation
+        pipe, hole chimney and the main bore shapes are converted into pipes
+        connected by junctions and having radiation condition.
 
-        1. the holes are localized on the main bore
+        1. the side elements (valves and holes) are localized on the main bore
         2. the entrance is associated to a source condition
         3. each shape constituing the main bore is converted in one or several\
             pipes if holes are localized on it
         4. the last end (the bell) is associated to a radiation condition
         """
-        # 1-localize the holes
-        self.position_holes = np.array([hole.position.get_value() for hole in
-                                        self.instrument_geometry.holes])
+        # 1-localize the holes and valves
+        self.side_elements = self._get_side_elements()
+        # the 2 end of the valve deviation pipe are not connected together and
+        # must be store somewhere.
+        self.unconnect_end_valves = dict.fromkeys([valve.label for valve in
+                                                self.instrument_geometry.valves])
 
         # 2-first shape "entrance"
         entrance_shape = self.instrument_geometry.main_bore_shapes[0]
@@ -377,9 +386,9 @@ class InstrumentPhysics:
 
         Parameters
         ----------
-        main_end_up : :py:class: `PipeEnd <openwind.continuous.netlist.PipeEnd>`
+        main_end_up : :py:class:`PipeEnd <openwind.continuous.netlist.PipeEnd>`
             The end of the "upstream" pipe.
-        main_end_down : :py:class: `PipeEnd <openwind.continuous.netlist.PipeEnd>`
+        main_end_down : :py:class:`PipeEnd <openwind.continuous.netlist.PipeEnd>`
             The end of the "downstream" pipe.
         junc_ID : str
             The name of the created `openwind.continuous.junction.JunctionSimple`
@@ -387,15 +396,17 @@ class InstrumentPhysics:
 
         """
         pos_up, _ = main_end_up.get_pipe().get_endpoints_position_value()
-        if any(self.position_holes == pos_up):
-            hole_ID = np.where(self.position_holes == pos_up)[0][0]
-            hole = self.instrument_geometry.holes[hole_ID]
-            if hole.position.is_variable():
+
+        side_at_junc = [elem for elem in self.side_elements if elem[0]==pos_up]
+        if len(side_at_junc) > 1:
+            raise ValueError('Two side elements have the same location: {}'.format(side_at_junc))
+        elif len(side_at_junc) == 1:
+            elem = side_at_junc[0]
+            if elem[2].is_variable():
                 warn_msg = ('It is impossible to vary the position of the {}'
-                            'placed at the junction of two '
-                            'pipes.').format(hole.label)
+                            'placed at the junction of two pipes.').format(elem[1])
                 warnings.warn(warn_msg)
-            self.__create_hole(hole, main_end_down, main_end_up)
+            self._connect_side_element(elem[1], main_end_down, main_end_up)
         else:
             label = 'junction_' + junc_ID
             if self.discontinuity_mass:
@@ -410,7 +421,7 @@ class InstrumentPhysics:
 
     def __create_main_bore_pipes(self, shape, pipeID):
         """Convert one shape of the main bore into one pipe or several if holes
-        are located on it.
+        or valves are located on it.
 
         Parameters
         ----------
@@ -427,17 +438,34 @@ class InstrumentPhysics:
             are still unconnected.
         """
         pos_min, pos_max = shape.get_endpoints_position()
-        # find the hole on the considered shape
-        holes_on_shape = ((self.position_holes > pos_min.get_value()) &
-                          (self.position_holes < pos_max.get_value()))
+
+        side_elem_on_shape = [elem for elem in self.side_elements
+                              if elem[0] > pos_min.get_value()
+                              and elem[0] < pos_max.get_value()]
+
         label = ('bore' + str(pipeID))
-        if not any(holes_on_shape):  # if no holes on the shape, no slicing
+        if len(side_elem_on_shape)>0:
+            pipe_ends = self.__slice_shape(side_elem_on_shape, shape, label)
+        else: # if no side elements on the shape, no slicing
             pipe_ends = self.__create_pipe(shape, label, pos_min, pos_max,
                                            main_bore=True)
-        else:
-            pipe_ends = self.__slice_shape(holes_on_shape, shape, label)
         return pipe_ends
 
+    def _get_side_elements(self):
+        """
+        Get all side elements (holes, valves (entry and reconnection)) and sort them
+        wr to their location on the main bore.
+        """
+        holes = [(hole.position.get_value(), hole, hole.position)
+                 for hole in self.instrument_geometry.holes]
+        valves = [(valve.position.get_value(), valve, valve.position)
+                  for valve in self.instrument_geometry.valves]
+        returns_valves = [(valve.reconnection_position.get_value(), valve.label,
+                           valve.reconnection_position)
+                          for valve in self.instrument_geometry.valves]
+        return sorted(holes + valves + returns_valves)
+
+
     def __create_pipe(self, shape, label, pos_min, pos_max, main_bore=False):
         """
         Create a pipe component of the graph.
@@ -447,14 +475,14 @@ class InstrumentPhysics:
 
         Parameters
         ----------
-        shape : :py:class: `DesignShape <openwind.design.design_shape.DesignShape>`
+        shape : :py:class:`DesignShape <openwind.design.design_shape.DesignShape>`
             The pipe shape.
         label : str
             The pipe name.
-        pos_min : :py:class: `DesignParameter <openwind.design.design_parameter.DesignParameter>`
+        pos_min : :py:class:`DesignParameter <openwind.design.design_parameter.DesignParameter>`
             The upstream end position of the pipe on the main bore axis (used
             for the temperature scale).
-        pos_max : :py:class: `DesignParameter <openwind.design.design_parameter.DesignParameter>`
+        pos_max : :py:class:`DesignParameter <openwind.design.design_parameter.DesignParameter>`
             The downstream end position of the pipe on the main bore axis (used
             for the temperature scale).
         main_bore : bool, optional
@@ -463,9 +491,9 @@ class InstrumentPhysics:
 
         Returns
         -------
-        end_up : :py:class: `PipeEnd <openwind.continuous.netlist.PipeEnd>`
+        end_up : :py:class:`PipeEnd <openwind.continuous.netlist.PipeEnd>`
             The upstream end of the created pipe.
-        end_down : :py:class: `PipeEnd <openwind.continuous.netlist.PipeEnd>`
+        end_down : :py:class:`PipeEnd <openwind.continuous.netlist.PipeEnd>`
             The downstream end of the created pipe.
 
         """
@@ -479,52 +507,79 @@ class InstrumentPhysics:
         end_up, end_down = self.netlist.add_pipe(pipe)
         return end_up, end_down
 
-    def __slice_shape(self, holes_on_shape, shape, label):
+    def __slice_shape(self, side_elem_on_shape, shape, label):
         """
-        Cut the shape at the position of the hole, create the corresponding
-        pipe and create the holes pipe and radiation condition
+        Cut the shape at the position of the side elements, create the corresponding
+        pipe and create the holes pipe and radiation condition and the valves
+        and the deviation pipes.
 
         Parameters
         ----------
-        holes_on_shape : np.array of boolean
-            If each hole is on the considered part of the main bore or not.
-        shape : :py:class: `DesignShape <openwind.design.design_shape.DesignShape>`
+        side_elem_on_shape : list of tuple
+            The side elements locate on the considered main bore part.
+            Each tuple contains 3 elements: the numerical value of the side
+            element location on the main bore, the element, and the
+            :py:class`DesignParameter` corresponding to its location.
+        shape : :py:class:`DesignShape <openwind.design.design_shape.DesignShape>`
             The main bore shape considered.
         label : str
             The name of the considered shape.
 
         Returns
         -------
-        upstream_end : :py:class: `PipeEnd <openwind.continuous.netlist.PipeEnd>`
+        upstream_end : :py:class:`PipeEnd <openwind.continuous.netlist.PipeEnd>`
             The main upstream end of the created pipes (still unconnected).
-        end_down : :py:class: `PipeEnd <openwind.continuous.netlist.PipeEnd>`
+        end_down : :py:class:`PipeEnd <openwind.continuous.netlist.PipeEnd>`
             The main downtream end of the created pipes (stil unconnected).
         """
+
+        side_pos, side_elem, design_pos = zip(*side_elem_on_shape)
+
         pos_min, pos_max = shape.get_endpoints_position()
-        index_in = np.argsort(self.position_holes[holes_on_shape])
         # the slicing positions (DesignParameters)
-        holes_in = [self.instrument_geometry.holes[nh] for nh in
-                    np.where(holes_on_shape)[0][index_in]]
-        x_cut = [pos_min] + [hole.position for hole in holes_in] + [pos_max]
+        x_cut = [pos_min] + list(design_pos) + [pos_max]
 
         shape_slice = ShapeSlice(shape, x_cut[0:2])
         label_slice = label + '_slice' + str(0)
         upstream_end, end_down = self.__create_pipe(shape_slice, label_slice,
                                                     x_cut[0], x_cut[1],
                                                     main_bore=True)
-        for p, hole in enumerate(holes_in):
+        for p, elem in enumerate(side_elem):
             shape_slice = ShapeSlice(shape, x_cut[p+1:p+3])
             label_slice = label + '_slice' + str(p+1)
             end_up, end_down_temp = self.__create_pipe(shape_slice,
                                                        label_slice,
                                                        x_cut[p+1], x_cut[p+2],
                                                        main_bore=True)
-            self.__create_hole(hole,
-                               end_down, end_up)
+            self._connect_side_element(elem, end_down, end_up)
             end_down = end_down_temp
 
         return upstream_end, end_down
 
+    def _connect_side_element(self, side_elem, end_down, end_up):
+        """
+        Connect the side element to the two ends of the main bore with
+        respect to its type.
+
+        Parameters
+        ----------
+        side_elem : :py:class:`Hole`, :py:class:`BrassValve` or str
+            The element to connect, if it is a string it corresponds to the
+            label of the valve the return of which must be connected.
+        end_down : :py:class:`PipeEnd`
+            The downstream end of the upstream pipe.
+        end_up : :py:class:`PipeEnd`
+            The upstream end of the downstream pipe
+
+        """
+        if type(side_elem) is Hole:
+            self.__create_hole(side_elem, end_down, end_up)
+        elif type(side_elem) is BrassValve:
+            self._connect_valve(side_elem.label, end_down, end_up, False)
+        elif type(side_elem) is str:
+            self._connect_valve(side_elem, end_up, end_down, True)
+
+
     def __hole_junction(self, junc_label, end_down, end_up, end_hole):
         """
         Set the T-joint junction between the hole and two main bore pipes.
@@ -536,11 +591,11 @@ class InstrumentPhysics:
         ----------
         junc_label : str
             The name of the junction.
-        end_down : :py:class: `PipeEnd <openwind.continuous.netlist.PipeEnd>`
+        end_down : :py:class:`PipeEnd <openwind.continuous.netlist.PipeEnd>`
             The "downstream" end of the "upstream" pipe.
-        end_up : :py:class: `PipeEnd <openwind.continuous.netlist.PipeEnd>`
+        end_up : :py:class:`PipeEnd <openwind.continuous.netlist.PipeEnd>`
             The "upstream" end of the "downstream" pipe.
-        end_hole : :py:class: `PipeEnd <openwind.continuous.netlist.PipeEnd>`
+        end_hole : :py:class:`PipeEnd <openwind.continuous.netlist.PipeEnd>`
             The "upstream" end of the pipe of the hole.
         """
         junction = JunctionTjoint(junc_label, self.scaling, self.convention,
@@ -561,9 +616,9 @@ class InstrumentPhysics:
         ----------
         hole : :py:class:`Hole <openwind.technical.instrument_geometry.Hole>`
             The hole considered.
-        end_down : :py:class: `PipeEnd <openwind.continuous.netlist.PipeEnd>`
+        end_down : :py:class:`PipeEnd <openwind.continuous.netlist.PipeEnd>`
             The "downstream" end of the "upstream" pipe.
-        end_up : :py:class: `PipeEnd <openwind.continuous.netlist.PipeEnd>`
+        end_up : :py:class:`PipeEnd <openwind.continuous.netlist.PipeEnd>`
             The "upstream" end of the "downstream" pipe.
 
         """
@@ -572,7 +627,7 @@ class InstrumentPhysics:
                                                         hole.position,
                                                         hole.position)
         # hole radiation
-        rad_label = 'rad_' + hole.label
+        rad_label = hole.label + '_radiation'
         hole_rad = radiation_model(self.rad_model(hole.label, end_hole_down),
                                      rad_label, self.scaling, self.convention)
         self.netlist.add_connector(hole_rad, end_hole_down)
@@ -581,6 +636,59 @@ class InstrumentPhysics:
         junc_label = 'junction_' + hole.label
         self.__hole_junction(junc_label, end_down, end_up, end_hole_up)
 
+
+    def _connect_valve(self, valve_label, end_main, end_default, is_reconnect):
+        """
+        Connect the considered end of the deviation pipe assiciated to the
+        brass valve.
+
+        If the deviation pipe does not exist yet, it is created, and the other
+        end of the pipe is stored in :py:attr:`unconnect_end_valves`
+
+        .. warning::
+            The switch is connected in different direction if it is the
+            entry or the reconnection of the deviation pipe!
+
+        Parameters
+        ----------
+        valve_label : str
+            The label of the considered brass valve.
+        end_down : :py:class:`PipeEnd <openwind.continuous.netlist.PipeEnd>`
+            The "downstream" end of the "upstream" pipe.
+        end_up : :py:class:`PipeEnd <openwind.continuous.netlist.PipeEnd>`
+            The "upstream" end of the "downstream" pipe.
+        is_reconnect : bool
+            If true, the connection convention for the switch is the one of the
+            reconnection switch.
+        """
+        # detect if the end already exists
+        end_to_connect = self.unconnect_end_valves.get(valve_label)
+        if not end_to_connect:
+            # due to the graph construction process, the return of the deviation
+            # pipe can be set before its begining, even if it is located after
+            # on the main bore.
+            valve = [valve for valve in self.instrument_geometry.valves
+                     if valve.label==valve_label][0]
+            end_valve_up, end_valve_down = self.__create_pipe(valve.shape,
+                                                              valve.label,
+                                                              valve.position,
+                                                              valve.reconnection_position)
+            if is_reconnect:
+                end_to_connect = end_valve_down
+                self.unconnect_end_valves[valve.label] = end_valve_up
+            else:
+                end_to_connect = end_valve_up
+                self.unconnect_end_valves[valve.label] = end_valve_down
+
+        if is_reconnect:
+            junc_label = valve_label + '_reconnection_switch'
+        else:
+            junc_label = valve_label + '_entry_switch'
+        junction = JunctionSwitch(junc_label, self.scaling, self.convention,
+                                  self.discontinuity_mass)
+        self.netlist.add_connector(junction, end_main, end_default, end_to_connect)
+
+
     def __check(self):
         """
         Check that the number of :py:class:`PhysicalRadiation \
