@@ -479,7 +479,7 @@ class Netlist:
         ..
             The following picture is only for the online documentation
             
-        .. image :: /images/convert_structure.png
+        .. image :: https://files.inria.fr/openwind/pictures/convert_structure.png
           :width: 1000
           :align: center
 
