@@ -1,19 +1,29 @@
 #!/usr/bin/env python3
 # -*- coding: utf-8 -*-
 
-__author__ = ["Juliette Chabassier", "Augustin Ernoult", "Olivier Geber",
-              "Alexis Thibault", "Tobias Van Baarsel"]
-__copyright__ = "Copyright 2020, Inria"
-__credits__ = ["Juliette Chabassier", "Augustin Ernoult", "Olivier Geber",
-               "Alexis Thibault", "Tobias Van Baarsel"]
-__license__ = "GPL 3.0"
-__version__ = "0.4"
-__email__ = "openwind-contact@inria.fr"
-__status__ = "Dev"
+# Copyright (C) 2019-2021, INRIA
+#
+# This file is part of Openwind.
+#
+# Openwind is free software: you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
+#
+# Openwind is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with Openwind.  If not, see <https://www.gnu.org/licenses/>.
+#
+# For more informations about authors, see the CONTRIBUTORS file
+
 import numpy as np
 
 from openwind.continuous import Physics
-from openwind.continuous import ThermoviscousLossless
+
 
 class Pipe:
     """A simple waveguide, described by its bore profile (shape) and its
@@ -21,7 +31,7 @@ class Pipe:
     equations (wave propagation) with or without losses. It may have a variable
     section and variable temperature.
 
-    In the frequential domain, the telegraphist equations are [1]_:
+    In the frequential domain, the telegraphist equations are [Tournemenne]_:
 
     .. math::
         Y_t(\\omega, x)  p + d_x u = 0, \\\\
@@ -29,10 +39,10 @@ class Pipe:
 
     where:
 
-    - \(u\) is the scaled flow
-    - \(p\) is the scaled pressure
-    - \(x\) is the scaled position ( \(0 \\leq x \\leq 1\) ),
-    - \(\\omega\) is the scaled angular frequency,
+    - :math:`u` is the scaled flow
+    - :math:`p` is the scaled pressure
+    - :math:`x` is the scaled position ( :math:`0 \\leq x \\leq 1` ),
+    - :math:`\\omega` is the scaled angular frequency,
 
     And the coefficients
 
@@ -44,13 +54,13 @@ class Pipe:
 
     where :
 
-    - \(S(x)\) is the cross section area at the \(x\) position
-    - \(\\rho\) is the air density at the \(x\) position
-    - \(c\) is the sound celerity at the \(x\) position
-    - \(\\ell\) is the length of the pipe
-    - \(Y_t^{\\ast}\) is the scaling coefficient of the pressure
-    - \(Z_v^{\\ast}\) is the scaling coefficient of the flow
-    - \(\\text{Losses}(\\omega, x)\) are the losses terms computed by the \
+    - :math:`S(x)` is the cross section area at the :math:`x` position
+    - :math:`\\rho` is the air density at the :math:`x` position
+    - :math:`c` is the sound celerity at the :math:`x` position
+    - :math:`\\ell` is the length of the pipe
+    - :math:`Y_t^{\\ast}` is the scaling coefficient of the pressure
+    - :math:`Z_v^{\\ast}` is the scaling coefficient of the flow
+    - :math:`\\text{Losses}(\\omega, x)` are the losses terms computed by the \
         `openwind.continuous.thermoviscous_models.ThermoviscousModel` object. \
         It must include the scaling chosen.
 
@@ -59,16 +69,16 @@ class Pipe:
 
     Parameters
     ----------
-    design_shape : openwind.design.design_shape.DesignShape
+    design_shape : :py:class:`DesignShape<openwind.design.design_shape.DesignShape>`
         The shape of the bore profile of the pipe
     temperature : float of callable
         The temperature allong the pipe (if float it is considered uniform)
     label : str
         the label of the pipe
-    scaling : openwind.continuous.scaling.Scaling
+    scaling : :py:class:`Scaling<openwind.continuous.scaling.Scaling>`
         object which knows the value of the coefficient used to scale the
         equations
-    losses : openwind.continuous.thermoviscous_models.ThermoviscousModel
+    losses : :py:class:`ThermoviscousModel<openwind.continuous.thermoviscous_models.ThermoviscousModel>`
         How to take into account thermoviscous losses
     convention : {'PH1', 'VH1'}, optional
         The basis functions for our finite elements must be of regularity
@@ -88,7 +98,7 @@ class Pipe:
 
     References
     -----------
-    .. [1] R. Tournemenne et J. Chabassier, "A comparison of a one-dimensional
+    .. [Tournemenne] R. Tournemenne et J. Chabassier, "A comparison of a one-dimensional
        finite element method and the transfer matrix method for the
        computation of wind music instrument impedance." Acta Acustica united
        with Acustica 105.5 (2019): 838-849.
@@ -105,7 +115,7 @@ class Pipe:
         self._spherical_waves = spherical_waves
         if spherical_waves:
             try:
-                design_shape.get_diff_shape_wr_x_norm(0.0)
+                design_shape.get_conicity_at(0.0)
             except NotImplementedError:
                 msg = ("Spherical waves not available for "
                        "shape type '{}'.")
@@ -128,30 +138,33 @@ class Pipe:
 
         Returns
         -------
-        openwind.continuous.thermoviscous_models.ThermoviscousModel
+        :py:class:`ThermoviscousModel<openwind.continuous.thermoviscous_models.ThermoviscousModel>`
             How to take into account thermoviscous losses.
+
         """
         return self._losses
 
     def get_scaling(self):
+
         """
         Return the scaling use for the equations coefficients.
 
         Returns
         -------
-        openwind.continuous.scaling.Scaling
-            object which knows the value of the coefficient used to scale the
-            equations
+        :py:class:`Scaling <openwind.continuous.scaling.Scaling>`
+           object which knows the value of the coefficient used to scale the
+           equations
+
         """
         return self._scaling
 
     def get_physics(self):
         """
-        Return the `openwind.continuous.physics.Physics` attached to the pipe.
+        Return the :py:class:`Physics<openwind.continuous.physics.Physics>` attached to the pipe.
 
         Returns
         -------
-        `openwind.continuous.physics.Physics`
+        :py:class:`Physics<openwind.continuous.physics.Physics>`
             This object computes the values of the physical constant (the air
             density, the sound celerity, etc.) along the pipe with respect to
             the temperature.
@@ -176,14 +189,14 @@ class Pipe:
         """
         return self._convention
 
-# %% ----------Geometry------------
+    # %% ----------Geometry------------
     def get_shape(self):
         """
         Return the shape of this pipe.
 
         Returns
         -------
-        openwind.design.design_shape.DesignShape
+        :py:class:`DesignShape<openwind.design.design_shape.DesignShape>`
         """
         return self._design_shape
 
@@ -240,7 +253,7 @@ class Pipe:
             The variation of radius with respect to (absolute) abscissa,
             measured at positions `x`.
         """
-        return self._design_shape.get_diff_shape_wr_x_norm(x) / self.get_length()
+        return self._design_shape.get_conicity_at(x)
 
     def get_section_at(self, x):
         """
@@ -270,29 +283,34 @@ class Pipe:
 
         """
         return self._design_shape.get_length()
-# %% ---------- PHYSICAL EQUATION ----------
-# ----- SCALING -----
+
+    # %% ---------- PHYSICAL EQUATION ----------
+
+    # ----- SCALING -----
+
     def get_scaling_flow(self):
         """Scaling coefficient associated to the flow equation.
 
-        This coefficient is used to nondimensionalize \(Z_v\):
+        This coefficient is used to nondimensionalize :math:`Z_v`:
 
         .. math::
             Z_v^{\\ast} = Z_c^{\\ast}  T^{\\ast}
 
         where:
 
-        - \(Z_c^{\\ast}\) : the reference caracteristic impedance
-        - \(T^{\\ast}\) : the reference time
+        - :math:`Z_c^{\\ast}` : the reference caracteristic impedance
+        - :math:`T^{\\ast}` : the reference time
+
+        See Also
+        --------
+        :py:class:`Scaling <openwind.continuous.scaling.Scaling>`
+
 
         Returns
         -------
         float
-            The coefficient \(Z_v^{\\ast}\)
+            The coefficient :math:`Z_v^{\\ast}`
 
-        See Also:
-        --------
-        `openwind.continuous.scaling.Scaling`
         """
         return self._scaling.get_impedance() * self._scaling.get_time()
 
@@ -300,28 +318,30 @@ class Pipe:
         """
         Scaling coefficient associated to the pressure equation.
 
-        This coefficient is used to nondimensionalize \(Y_t\):
+        This coefficient is used to nondimensionalize :math:`Y_t`:
 
         .. math::
             Y_t^{\\ast} = \\frac{T^{\\ast}}{Z_c^{\\ast}}
 
         where:
 
-        - \(Z_c^{\\ast}\) : the reference caracteristic impedance
-        - \(T^{\\ast}\) : the reference time
+        - :math:`Z_c^{\\ast}` : the reference caracteristic impedance
+        - :math:`T^{\\ast}` : the reference time
+
+        See Also
+        --------
+        :py:class:`Scaling <openwind.continuous.scaling.Scaling>`
+
 
         Returns
         -------
         float
-            The coefficient \(Y_t^{\\ast}\)
+            The coefficient :math:`Y_t^{\\ast}`
 
-        See Also:
-        --------
-        `openwind.continuous.scaling.Scaling`
         """
         return self._scaling.get_time() / self._scaling.get_impedance()
 
-# ----- TELEGRAPHISTE EQUATION -----
+    # ----- TELEGRAPHISTE EQUATION -----
     def get_coef_flow_at(self, x):
         """
         Coefficient without losses associated to the flow.
@@ -329,13 +349,22 @@ class Pipe:
         Gives the values of the part of coefficient associated to the flow in
         the scaled telegraphist equations independant of the frequency and
         without losses at the normalized position "x":
-        $$ \\frac{1}{Z_v^{\\ast}} \\frac{\\rho}{S} \\ell $$
+
+        .. math::
+             \\frac{1}{Z_v^{\\ast}} \\frac{\\rho}{S} \\ell
 
         .. warning::
             This method is used when the equations solved can not be written
-            with a \(Z_v\) e.g. in the temporal domain and/or when the losses
+            with a :math:`Z_v` e.g. in the temporal domain and/or when the losses
             are include trough auxillary variables.
 
+        See Also
+        --------
+        :py:class:`FrequentialPipeDiffusiveRepresentation \
+        <openwind.frequential.frequential_pipe_diffusive_representation.\
+        FrequentialPipeDiffusiveRepresentation>`
+        :py:class:`TemporalPipe <openwind.temporal.tpipe.TemporalPipe>`
+
         Parameters
         ----------
         x : float or array(float)
@@ -343,13 +372,9 @@ class Pipe:
 
         Returns
         -------
-        float or array(float), same shape as x
+        coef : float or array(float), same shape as x
             The value of the coefficient at positions `x`.
 
-        See Also :
-        ----------
-        `openwind.frequential.frequential_pipe_diffusive_representation.FrequentialPipeDiffusiveRepresentation`
-        `openwind.temporal.tpipe.TemporalPipe`
         """
         section = np.pi * (self.get_radius_at(x))**2
         rho = self._physics.rho(x)
@@ -365,13 +390,22 @@ class Pipe:
         Gives the values of the part independant of the frequency of the
         coefficient associated to the pressure in the scaled telegraphist
         equations without losses at the normalized position "x":
-        $$ \\frac{1}{Y_t^{\\ast}} \\frac{S}{\\rho c^2} \\ell $$
+
+        .. math::
+             \\frac{1}{Y_t^{\\ast}} \\frac{S}{\\rho c^2} \\ell
 
         .. warning::
             This method is used when the equations solved can not be written
-            with a \(Y_t\) e.g. in the temporal domain and/or when the losses
+            with a :math:`Y_t` e.g. in the temporal domain and/or when the losses
             are include trough auxillary variables.
 
+        See Also
+        --------
+        :py:class:`FrequentialPipeDiffusiveRepresentation \
+        <openwind.frequential.frequential_pipe_diffusive_representation.\
+        FrequentialPipeDiffusiveRepresentation>`
+        :py:class:`TemporalPipe <openwind.temporal.tpipe.TemporalPipe>`
+
         Parameters
         ----------
         x : float or array(float)
@@ -382,10 +416,6 @@ class Pipe:
         float or array(float), same shape as x
             The value of the coefficient at positions `x`.
 
-        See Also :
-        ----------
-        `openwind.frequential.frequential_pipe_diffusive_representation.FrequentialPipeDiffusiveRepresentation`
-        `openwind.temporal.tpipe.TemporalPipe`
         """
         section = np.pi * (self.get_radius_at(x))**2
         rho_c2 = self._physics.rho(x) * self._physics.c(x)**2
@@ -394,20 +424,22 @@ class Pipe:
             coef *= np.sqrt(1 + self.get_conicity_at(x)**2)
         return coef
 
-# ------ FULL FREQUENTIAL COEFFICIENTS ------
+    # ------ FULL FREQUENTIAL COEFFICIENTS ------
 
     def get_Zv_at(self, x, omegas_scaled):
         """
         Flow coefficent in the frequential equation.
 
-        Give the value of the flow coefficient \(Z_v\) estimated at the given
+        Give the value of the flow coefficient :math:`Z_v` estimated at the given
         normalized positions for the given normalized angular frequencies:
-        $$Z_v(\\omega, x) = j \\omega \\left( \\frac{1}{Z_v^{\\ast}} \
-            \\frac{\\rho}{S} \\ell + \\text{Losses}_u(\\omega, x) \\right)$$
+
+        .. math::
+            Z_v(\\omega, x) = j \\omega \\left( \\frac{1}{Z_v^{\\ast}} \
+            \\frac{\\rho}{S} \\ell + \\text{Losses}_u(\\omega, x) \\right)
 
 
         .. warning::
-            The losses must include the scaling term \(Z_v^{\\ast}\)
+            The losses must include the scaling term :math:`Z_v^{\\ast}`
 
         Parameters
         ----------
@@ -431,13 +463,15 @@ class Pipe:
         """
         Pressure coefficent in the frequential equation.
 
-        Give the value of the pressure coefficient \(Y_t\) estimated at the given
+        Give the value of the pressure coefficient :math:`Y_t` estimated at the given
         normalized positions for the given normalized angular frequencies:
-        $$Y_t(\\omega, x) = j \\omega \\left( \\frac{1}{Y_t^{\\ast}} \
-        \\frac{S}{\\rho c^2} \\ell + \\text{Losses}_p(\\omega, x)  \\right)$$
+
+        .. math::
+            Y_t(\\omega, x) = j \\omega \\left( \\frac{1}{Y_t^{\\ast}} \
+        \\frac{S}{\\rho c^2} \\ell + \\text{Losses}_p(\\omega, x)  \\right)
 
         .. warning::
-            The losses must include the scaling term \(Y_t^{\\ast}\)
+            The losses must include the scaling term :math:`Y_t^{\\ast}`
 
         Parameters
         ----------
@@ -458,7 +492,7 @@ class Pipe:
                                                           omegas_scaled)
         return 1j*omegas_scaled[:, np.newaxis]*(Yt_wo_losses + losses_p)
 
-# %% ---------- DIFFERENTIATION ----------
+    # %% ---------- DIFFERENTIATION ----------
 
     def get_diff_length(self, diff_index):
         """
@@ -487,11 +521,12 @@ class Pipe:
 
         Parameters
         ----------
-         x : float or array(float)
+        x : float or array(float)
             Normalized positions along the pipe within 0 and 1.
         diff_index : int
             The index of the desire design parameter in the \
-            `openwind.design.design_parameter.OptimizationParameters`.
+            :py:class:`OptimizationParameters \
+            <openwind.design.design_parameter.OptimizationParameters>`.
 
         Returns
         -------
@@ -512,7 +547,8 @@ class Pipe:
         ----------
         diff_index : int
             The index of the desire design parameter in the \
-            `openwind.design.design_parameter.OptimizationParameters`.
+            :py:class:`OptimizationParameters \
+            <openwind.design.design_parameter.OptimizationParameters>`.
 
         Returns
         -------
@@ -540,7 +576,8 @@ class Pipe:
             Normalized positions along the pipe within 0 and 1.
         diff_index : int
             The index of the desire design parameter in the \
-            `openwind.design.design_parameter.OptimizationParameters`.
+            :py:class:`OptimizationParameters \
+            <openwind.design.design_parameter.OptimizationParameters>`.
 
         Returns
         -------
@@ -549,11 +586,17 @@ class Pipe:
         """
         radius = self.get_radius_at(x)
         diff_radius = self.get_diff_radius_at(x, diff_index)
-        partial_wr_radius = -2*diff_radius/radius * self.get_coef_flow_at(x)
+        coef_flow = self.get_coef_flow_at(x)
+        partial_wr_radius = -2*diff_radius/radius * coef_flow
 
         diff_length = self.get_diff_length(diff_index)
-        partial_wr_length = diff_length/self.get_length() * self.get_coef_flow_at(x)
-        return partial_wr_radius + partial_wr_length
+        partial_wr_length = diff_length/self.get_length() * coef_flow
+        d_coef = partial_wr_radius + partial_wr_length
+        if self._spherical_waves:
+            conicity = self.get_conicity_at(x)
+            d_coef += (coef_flow*conicity/(1 + conicity**2)
+                        * self._design_shape.get_diff_conicity_at(x, diff_index))
+        return d_coef
 
     def get_diff_coef_pressure_at(self, x, diff_index):
         """
@@ -570,7 +613,8 @@ class Pipe:
             Normalized positions along the pipe within 0 and 1.
         diff_index : int
             The index of the desire design parameter in the \
-            `openwind.design.design_parameter.OptimizationParameters`.
+            :py:class:`OptimizationParameters \
+            <openwind.design.design_parameter.OptimizationParameters>`.
 
         Returns
         -------
@@ -578,30 +622,37 @@ class Pipe:
             The value of the differential at the given positions.
         """
         radius = self.get_radius_at(x)
+        coef_pres = self.get_coef_pressure_at(x)
         diff_radius = self.get_diff_radius_at(x, diff_index)
-        partial_wr_radius = 2*diff_radius/radius * self.get_coef_pressure_at(x)
+        partial_wr_radius = 2*diff_radius/radius * coef_pres
 
         diff_length = self.get_diff_length(diff_index)
-        partial_wr_length = diff_length/self.get_length() * self.get_coef_pressure_at(x)
-        return partial_wr_radius + partial_wr_length
+        partial_wr_length = diff_length/self.get_length() * coef_pres
+        d_coef = partial_wr_radius + partial_wr_length
+        if self._spherical_waves:
+            conicity = self.get_conicity_at(x)
+            d_coef += (coef_pres*conicity/(1 + conicity**2)
+                       * self._design_shape.get_diff_conicity_at(x, diff_index))
+        return d_coef
 
     def get_diff_Zv_at(self, x, omegas_scaled, diff_index):
         """
         Differentiate the flow coefficient with losses in frequential equation.
 
-        Give the differential of the coefficient \(Z_v\) in the telegraphist
+        Give the differential of the coefficient :math:`Z_v` in the telegraphist
         equations with respect to the designated optimization parameter at the
         given normalized positions and normalized angular frequencies.
 
         Parameters
         ----------
-         x : float or array(float)
+        x : float or array(float)
             Normalized positions along the pipe within 0 and 1.
         omegas_scaled : float or array(float)
             Normalized angular frequencies.
         diff_index : int
             The index of the desire design parameter in the \
-            `openwind.design.design_parameter.OptimizationParameters`.
+            :py:class:`OptimizationParameters \
+            <openwind.design.design_parameter.OptimizationParameters>`.
 
         Returns
         -------
@@ -620,19 +671,20 @@ class Pipe:
         """
         Differentiate the pressure coefficient with losses.
 
-        Give the differential of the coefficient \(Y_t\) in the telegraphist
+        Give the differential of the coefficient :math:`Y_t` in the telegraphist
         equations with respect to the designated optimization parameter at the
         given normalized positions and normalized angular frequencies.
 
         Parameters
         ----------
-         x : float or array(float)
+        x : float or array(float)
             Normalized positions along the pipe within 0 and 1.
         omegas_scaled : float or array(float)
             Normalized angular frequencies.
         diff_index : int
             The index of the desire design parameter in the \
-            `openwind.design.design_parameter.OptimizationParameters`.
+            :py:class:`OptimizationParameters \
+            <openwind.design.design_parameter.OptimizationParameters>`.
 
         Returns
         -------
