@@ -82,7 +82,7 @@ class FrequentialJunctionDiscontinuity(FrequentialComponent):
 
     def __get_masses(self):
         r1, r2, rho = self.__get_physical_params()
-        mass = self.junc.compute_masse(r1, r2, rho)
+        mass = self.junc.compute_mass(r1, r2, rho)
         return mass
 
     def get_number_dof(self):
@@ -99,7 +99,8 @@ class FrequentialJunctionDiscontinuity(FrequentialComponent):
 
     def get_contrib_indep_freq(self):
         assembled_interaction_matrix = ssp.lil_matrix((self.ntot_dof,
-                                                       self.ntot_dof))
+                                                       self.ntot_dof),
+                                                      dtype='complex128')
         interaction = [-1, 1]
         for i in range(len(self.ends)):
             f_pipe_end = self.ends[i]
