@@ -32,7 +32,7 @@ import openwind.impedance_tools as tools
 from openwind.design import Cone
 from openwind.technical import Fingering
 from openwind.continuous import (PhysicalRadiation, Excitator, Flow,
-                                 JunctionTjoint, JunctionSimple,
+                                 JunctionTjoint, JunctionSimple, JunctionSwitch,
                                  ThermoviscousLossless, JunctionDiscontinuity,
                                  ThermoviscousDiffusiveRepresentation,
                                  RadiationPerfectlyOpen)
@@ -40,6 +40,7 @@ from openwind.frequential import (FrequentialPipeFEM, FrequentialRadiation,
                                   FrequentialJunctionTjoint,
                                   FrequentialJunctionSimple,
                                   FrequentialJunctionDiscontinuity,
+                                  FrequentialJunctionSwitch,
                                   FrequentialSource,
                                   FrequentialInterpolation,
                                   FrequentialPipeDiffusiveRepresentation,
@@ -105,10 +106,11 @@ class FrequentialSolver:
                  note=None, compute_method='FEM', **discr_params):
         self.netlist = instru_physics.netlist
         self.discr_params = discr_params
-        frequencies = self._check_frequencies(frequencies)
-        self.frequencies = frequencies
         self.diffus_repr_var = diffus_repr_var   # When using 'diffrepr+'
         self.compute_method = compute_method   # FEM, TMM, or hybrid?
+
+        frequencies = self._check_frequencies(frequencies, compute_method)
+        self.frequencies = frequencies
         self._convert_frequential_components()
         self._organize_components()
         self._construct_matrices_pipes()
@@ -159,7 +161,7 @@ class FrequentialSolver:
         return self.impedance
 
     @staticmethod
-    def _check_frequencies(frequencies):
+    def _check_frequencies(frequencies, compute_method):
         """
         Check that all the frequencies are positive and that it is an array
 
@@ -174,6 +176,10 @@ class FrequentialSolver:
         frequencies = np.array(frequencies)
         if np.any(frequencies <= 0):
             raise ValueError('The frequencies must be strictly positive!')
+        if compute_method != 'TMM' and max(frequencies)/min(frequencies)>1000:
+            warnings.warn('The frequency range is too big (fmax/fmin={:.0f}, advised: <1000). With FEM the use of'
+                          'the same mesh for such range of frequencies can '
+                          'induce numerical issue.'.format(max(frequencies)/min(frequencies)))
         return frequencies
 
     def _set_note(self, note):
@@ -212,7 +218,7 @@ class FrequentialSolver:
             The new frequency axis.
 
         """
-        self.frequencies = self._check_frequencies(frequencies)
+        self.frequencies = self._check_frequencies(frequencies, self.compute_method)
         self._organize_components()
         self._construct_matrices_pipes()
         self._construct_matrices()
@@ -311,6 +317,8 @@ class FrequentialSolver:
             return FrequentialJunctionSimple(connector, ends)
         elif isinstance(connector, JunctionDiscontinuity):
             return FrequentialJunctionDiscontinuity(connector, ends)
+        elif isinstance(connector, JunctionSwitch):
+            return FrequentialJunctionSwitch(connector, ends)
 
         raise ValueError("Could not convert connector %s" % str(connector))
 
@@ -369,10 +377,6 @@ class FrequentialSolver:
         (Ah_conect_nodiag, Ah_conect_diags,
          Lh_conect) = self._construct_matrices_of(self.f_connectors)
         self.Ah_nodiag = self.Ah_pipes_nodiag + Ah_conect_nodiag
-        # for numerical purpose, it is interesting to put arbitrary diag values
-        with warnings.catch_warnings():
-            warnings.simplefilter("ignore", SparseEfficiencyWarning)
-            self.Ah_nodiag.setdiag(1)
         self.Ah_diags = self.Ah_pipes_diags + Ah_conect_diags
         self.Lh = self.Lh_pipes + Lh_conect
 
@@ -457,11 +461,12 @@ class FrequentialSolver:
         tracker = SimulationTracker(len(self.frequencies),
                                     display_enabled=enable_tracker_display)
 
-        Ah = self.Ah_nodiag
+        Ah, ind_diag = self._initialize_Ah_diag()
         # the loop on the frequency
         for cpt in range(len(self.frequencies)):
             # the frequency dependant part of Ah is only on its diagonal
-            Ah.setdiag(self.Ah_diags[:, cpt])
+            # Ah.setdiag(self.Ah_diags[:, cpt])
+            Ah.data[ind_diag] = self.Ah_diags[:, cpt]
             # solve the problem
             Uh = spsolve(Ah, self.Lh, permc_spec='COLAMD')
             tracker.update()
@@ -488,6 +493,31 @@ class FrequentialSolver:
                 self.pressure = np.array(interp_L2) * self.scaling.get_scaling_pressure()
                 self.flow = np.array(interp_H1) * self.scaling.get_scaling_flow()
 
+    def _initialize_Ah_diag(self):
+        """
+        Initialize diag of the sparse matrix Ah and return the corresponding indices
+
+        for numerical purpose, it is interesting to initiate diag values and
+        get the corresponding index in the "data" vector
+
+        Returns
+        -------
+        Ah : csr sparse matrix
+            The matrix Ah with abritrary values in diagonal
+        ind_diag : array of int
+            The indices corresponding to the diagonal values in the "data" attributes
+            of the sparse matrix.
+        """
+        if not all(np.isfinite(self.Ah_nodiag.data)):
+            raise ValueError('The matrix Ah contains non-finite value(s) (inf or NaN).')
+        Ah = self.Ah_nodiag.tocsc()
+        with warnings.catch_warnings():
+            warnings.simplefilter("ignore", SparseEfficiencyWarning)
+            Ah.setdiag(np.NaN)
+        ind_diag = np.where(np.isnan(Ah.data))[0]
+        return Ah, ind_diag
+
+
     # %% several notes
 
     def get_flow_pressure_several_notes(self, notes, f_interest, interp_grid='original'):
@@ -752,7 +782,7 @@ class FrequentialSolver:
 
     # %% --- Post-processing functions ---
 
-    def write_impedance(self, filename, normalize=False):
+    def write_impedance(self, filename, column_sep=' ', normalize=False):
         """
         Write the impedance in a file.
 
@@ -766,6 +796,8 @@ class FrequentialSolver:
         filename : string
             The name of the file in which is written the impedance (with the
             extension).
+        column_sep : str, optional
+            The column separator. Default is ' ' (space)
         normalize : bool, optional
             Normalize or not the impedance by the input characteristic
             impedance. The default is False.
@@ -774,7 +806,7 @@ class FrequentialSolver:
             impedance = self.impedance/self.get_ZC_adim()
         else:
             impedance = self.impedance
-        tools.write_impedance(self.frequencies, impedance, filename)
+        tools.write_impedance(self.frequencies, impedance, filename, column_sep)
 
     def resonance_frequencies(self, k=5):
         """
