@@ -73,9 +73,20 @@ class ImpedanceComputation:
     fs : numpy.array
         Frequencies at which to compute the impedance.
 
-    files : text files
-        Geometry of the instrument. For more information on the file
-        structures. See also : :py:class:`InstrumentGeometry \
+    main_bore : str or list
+        filename or list of data respecting the file format with the
+        main bore geometry. See also : :py:class:`InstrumentGeometry \
+        <openwind.technical.instrument_geometry.InstrumentGeometry>`
+    holes_or_vales : str or list, optional
+        filename or list of data respecting the file format, with the
+        holes and/or valves geometries. The default is None corresponding to
+        an instrument without hole or valve. See also : :py:class:`InstrumentGeometry \
+        <openwind.technical.instrument_geometry.InstrumentGeometry>`
+    fingering_chart : str or list, optional
+        filename or list of data respecting the file format, indicating the
+        fingering chart in accordance with the holes and/or valves. The default
+        is None corresponding to no fingering (everything open).
+        See also : :py:class:`InstrumentGeometry \
         <openwind.technical.instrument_geometry.InstrumentGeometry>`
 
     player : :py:class:`Player <openwind.technical.player.Player>`, optional
@@ -180,7 +191,8 @@ class ImpedanceComputation:
     adapted to the frequency max([frequencies, FMIN_disc])
     """
 
-    def __init__(self, frequencies, *files, player = None, temperature=None,
+    def __init__(self, frequencies, main_bore, holes_valves=None,
+                 fingering_chart=None, player = None, temperature=None,
                  losses=True,
                  compute_method='FEM', l_ele=None, order=None, nb_sub=1, note=None,
                  convention='PH1', nondim=True, radiation_category='unflanged',
@@ -204,7 +216,8 @@ class ImpedanceComputation:
         else:
             diff_repr_vars = False
 
-        self.__instrument_geometry = InstrumentGeometry(*files)
+        self.__instrument_geometry = InstrumentGeometry(main_bore, holes_valves,
+                                                        fingering_chart)
         self.__instru_physics = InstrumentPhysics(self.__instrument_geometry, temperature, player=player,
                                       losses=losses,
                                       radiation_category=radiation_category,
@@ -321,7 +334,7 @@ class ImpedanceComputation:
         """
         self.__freq_model.plot_impedance(**kwargs)
 
-    def write_impedance(self, filename, normalize=False):
+    def write_impedance(self, filename, column_sep=' ', normalize=False):
         """
         Write the impedance in a file.
 
@@ -335,11 +348,14 @@ class ImpedanceComputation:
         filename : string
             The name of the file in which is written the impedance (with the
             extension).
+        column_sep : str, optional
+            The column separator. Default is ' ' (space)
         normalize : bool, optional
             Normalize or not the impedance by the input characteristic
             impedance. The default is False.
+
         """
-        self.__freq_model.write_impedance(filename, normalize)
+        self.__freq_model.write_impedance(filename, column_sep, normalize)
 
     def resonance_frequencies(self, k=5):
         """
