@@ -27,6 +27,7 @@ import matplotlib.pyplot as plt
 from scipy.sparse import coo_matrix, vstack, csr_matrix
 from scipy.sparse.linalg import splu
 from scipy.optimize import minimize, least_squares, Bounds
+import warnings
 
 from openwind.frequential import FrequentialSolver, FrequentialInterpolation
 from openwind.algo_optimization import homemade_minimization, print_cost
@@ -187,6 +188,8 @@ class InverseFrequentialResponse(FrequentialSolver):
         for impedance in target_impedances:
             if len(impedance.shape) == 1:
                 impedance = np.array([impedance[:]])
+            if np.mean(np.abs(impedance))>100:
+                warnings.warn('The target impedance does not seem normalized (mean magnitude > 100; it should be around 1)')
             targets_list.append(impedance)
         assert len(notes) == len(targets_list)
         assert targets_list[0].shape[1] == len(self.frequencies)
@@ -407,11 +410,12 @@ class InverseFrequentialResponse(FrequentialSolver):
         Nres = self._target.shape[0]*2  # observation points *2 (real and imag)
         residuals = np.zeros(Nres*self._target.shape[1])
         jacobian = np.zeros((Nres*self._target.shape[1], Nderiv))
-        Ah = self.Ah_nodiag.tocsc()
+        Ah, ind_diag = self._initialize_Ah_diag()
         Lh = self.Lh.toarray()
         self.__computedAH()
         for nf in range(len(self.frequencies)):
-            Ah.setdiag(self.Ah_diags[:, nf])
+            # Ah.setdiag(self.Ah_diags[:, nf])
+            Ah.data[ind_diag] = self.Ah_diags[:, nf]
             Ahlu = splu(Ah, permc_spec='COLAMD')
             Uh = Ahlu.solve(Lh)[:, 0]
             residuals[Nres*nf:Nres*(nf+1)] = self.__compute_residu(Uh, nf)
@@ -474,7 +478,8 @@ class InverseFrequentialResponse(FrequentialSolver):
 
         residu = np.zeros((self._target.shape[0]*2, self._target.shape[1]))
 
-        Ah = self.Ah_nodiag.tocsc()
+        # Ah = self.Ah_nodiag.tocsc()
+        Ah, ind_diag = self._initialize_Ah_diag()
         Lh = self.Lh.toarray()
 
         if grad_type == 'frechet' or grad_type == 'adjoint':
@@ -483,7 +488,8 @@ class InverseFrequentialResponse(FrequentialSolver):
             if grad_type == 'frechet':
                 hessian = np.zeros([Nderiv, Nderiv])
         for ind_freq in range(len(self.frequencies)):
-            Ah.setdiag(self.Ah_diags[:, ind_freq])
+            # Ah.setdiag(self.Ah_diags[:, ind_freq])
+            Ah.data[ind_diag] = self.Ah_diags[:, ind_freq]
             Ahlu = splu(Ah, permc_spec='COLAMD')
             Uh = Ahlu.solve(Lh)[:, 0]
             residu[:, ind_freq] = self.__compute_residu(Uh, ind_freq)
@@ -882,7 +888,8 @@ class InverseFrequentialResponse(FrequentialSolver):
 
         grad_observation = np.zeros((Nfreq, Nderiv), dtype='complex')
         observation = np.zeros(Nfreq, dtype='complex')
-        Ah = self.Ah_nodiag.tocsc()
+        # Ah = self.Ah_nodiag.tocsc()
+        Ah, ind_diag = self._initialize_Ah_diag()
         Lh = self.Lh.toarray()
         if interp:
             interpolation = FrequentialInterpolation(self, pipes_label,
@@ -899,7 +906,8 @@ class InverseFrequentialResponse(FrequentialSolver):
         self.__computedAH()
 
         for ind_freq in range(Nfreq):
-            Ah.setdiag(self.Ah_diags[:, ind_freq])
+            # Ah.setdiag(self.Ah_diags[:, ind_freq])
+            Ah.data[ind_diag] = self.Ah_diags[:, ind_freq]
             Ahlu = splu(Ah, permc_spec='COLAMD')
             Uh = Ahlu.solve(Lh)[:, 0]
 
