@@ -52,30 +52,31 @@ class FingeringChart:
     ----------
     note_names : list of str, optional
         The list of the note names. The default is an empty list.
-    hole_names : list of str, optional
-        The list of the hole names. The default is an empty list.
+    side_comp_labels : list of str, optional
+        The list of the side components names (holes and valves). The default is
+        an empty list.
     chart : np.array, optional
-        Opening factor for each hole. 0 is closed, 1 is open. Its dimension
-        must be consistent with the note_names and hole_names length.
-        The default is an empty list.
-    other_holes : list of str, optional
-        Names of holes that exist in the instrument, but not indicated in the
+        Opening factor for each hole or valve. 0 is closed/depressed, 1 is
+        open/raised. Its dimension must be consistent with the note_names
+        and side_comp_labels length. The default is an empty list.
+    other_side_comp : list of str, optional
+        Names of holes or valves that exist in the instrument, but not indicated in the
         fingering chart because always remain open.
     """
 
-    def __init__(self, note_names=[], hole_labels=[], chart=[],
-                 other_holes=[]):
+    def __init__(self, note_names=[], side_comp_labels=[], chart=[],
+                 other_side_comp=[]):
         chart = np.array(chart)
-        if (chart.shape != (len(hole_labels), len(note_names))
+        if (chart.shape != (len(side_comp_labels), len(note_names))
             and chart.shape != (0,)):
             raise ValueError('The chart dimensions are not consistent with the'
                              ' number of holes and notes.')
         if np.any(chart < 0) or np.any(chart > 1):
             raise ValueError('The chart coefficient must be within 0 and 1')
         self._notes = note_names
-        self._holes = hole_labels
+        self._side_comp = side_comp_labels
         self._chart = chart
-        self._other_holes = other_holes
+        self._other_side_comp = other_side_comp
         self._current_note = 'None'
 
     def fingering_of(self, note):
@@ -129,7 +130,7 @@ class FingeringChart:
             col_names = ['label'] + self._notes
             chart = [['o' if s else 'x' for s in row] for row in self._chart]
             rows = [[hole] + list(chart_row)
-                    for hole, chart_row in zip(self._holes, chart)
+                    for hole, chart_row in zip(self._side_comp, chart)
                     ]
             return tabulate(rows, col_names)
         else:
@@ -164,17 +165,17 @@ class FingeringChart:
         if len(ax) < 1:
             ax.append(fig.add_subplot(1, 1, 1))
 
-        holes_tot = self._holes + self._other_holes
+        side_tot = self._side_comp + self._other_side_comp
         x = np.arange(0, len(self._notes))
-        y = np.arange(0, len(holes_tot))
+        y = np.arange(0, len(side_tot))
         xx, yy = np.meshgrid(x, y)
 
         fing_array = list()
         for note in self._notes:
             finger = self.fingering_of(note)
             one_fing = list()
-            for hole in holes_tot:
-                one_fing += ([bool(finger.is_hole_open(hole))])
+            for side in side_tot:
+                one_fing += ([bool(finger.is_side_comp_open(side))])
             fing_array.append(one_fing)
         fing_array = np.array(fing_array).T
         ax[0].plot(xx[fing_array], yy[fing_array], 'o', **kwargs)
@@ -183,8 +184,8 @@ class FingeringChart:
         if not figure:
             ax[0].set_xticks(range(len(self._notes)))
             ax[0].set_xticklabels(self._notes)
-            ax[0].set_yticks(range(len(holes_tot)))
-            ax[0].set_yticklabels(holes_tot)
+            ax[0].set_yticks(range(len(side_tot)))
+            ax[0].set_yticklabels(side_tot)
             ax[0].invert_yaxis()
             ax[0].grid(True)
 
@@ -194,8 +195,8 @@ class Fingering:
     """
     One fingering position.
 
-    To each hole, associates its opening factor
-    (0 if closed, 1 if open).
+    To each side component (hole or valve), associates its opening factor
+    (0 if closed/depressed, 1 if open/raised).
 
     Parameters
     ----------
@@ -203,10 +204,10 @@ class Fingering:
     <openwind.technical.fingering_chart.FingeringChart>`
         The fingering chart from which this fingering derives.
     opening_factors: numpy.array
-        The array of opening factor for each hole.
+        The array of opening factor for each side component.
     """
     def __init__(self, chart, opening_factors):
-        assert len(chart._holes) == len(opening_factors)
+        assert len(chart._side_comp) == len(opening_factors)
         self._chart = chart
         self._opening_factors = opening_factors
 
@@ -216,8 +217,8 @@ class Fingering:
         Modify the radiation condition of a set of components by following the
         fingering.
         Assumes the key of the correct radiation components is
-        'rad_{hole_label}', and that it has a method
-        `set_opening_factor(factor)`.
+        '{hole_label}_radiation', '{valve_label}_entry_switch' or '{valve_label}_reconnection_switch'
+        and that it has a method `set_opening_factor(factor)`.
 
         Parameters
         ----------
@@ -228,32 +229,44 @@ class Fingering:
          <openwind.temporal.tcomponent.TemporalComponent>`
             List of components at which are apllied the fingering.
         """
-        for hole_label, factor in zip(self._chart._holes, self._opening_factors):
-            rad_label = 'rad_' + hole_label
-            components[rad_label].set_opening_factor(factor)
+
+        for side_comp_label, factor in zip(self._chart._side_comp, self._opening_factors):
+            # rad_label = 'rad_' + hole_label
+            comp_labels = [key for key in components.data.keys()
+                          if key == (side_comp_label + '_radiation')
+                          or key == (side_comp_label + '_reconnection_switch')
+                          or key == (side_comp_label + '_entry_switch')]
+            assert len(comp_labels)>0 and len(comp_labels)<3
+            for label in comp_labels:
+                components[label].set_opening_factor(factor)
+
+    def is_side_comp_open(self, side_comp_label):
+      """
+      Gives the opening state of a side component (hole or valve).
+
+      Parameters
+      ----------
+      side_comp_label : string
+          The name of the side component considered.
+
+      Returns
+      -------
+      boolean or float within 0 and 1
+          The opening factor of the side component.
+
+      """
+      if side_comp_label in self._chart._other_side_comp:
+          return True
+      return self._opening_factors[self._chart._side_comp.index(side_comp_label)]
+
 
     def is_hole_open(self, hole_label):
-        """
-        Gives the opening state of a hole.
+        return self.is_side_comp_open(hole_label)
 
-        Parameters
-        ----------
-        hole_label : string
-            The name of the hole considered.
-
-        Returns
-        -------
-        boolean or float within 0 and 1
-            The opening factor of the hole.
-
-        """
-        if hole_label in self._chart._other_holes:
-            return True
-        return self._opening_factors[self._chart._holes.index(hole_label)]
 
     def __str__(self):
         return tabulate([['o' if s else 'x' for s in self._opening_factors]],
-                        self._chart._holes)
+                        self._chart._side_comp)
 
     def mix(self, other, factor):
         """Create intermediate fingering between this and other.
