@@ -32,7 +32,7 @@ import matplotlib.pyplot as plt
 from openwind.design import (FixedParameter, OptimizationParameters,
                              VariableParameter,
                              VariableHolePosition, VariableHoleRadius)
-from openwind.design import Spline, Cone, Bessel, Circle, Exponential
+from openwind.design import Spline, Cone, Bessel, Circle, Exponential, ShapeSlice
 from openwind.technical import FingeringChart
 from openwind.technical.fingering_chart import tabulate
 
@@ -57,24 +57,63 @@ class Hole:
         self.position = position
         self.label = label
 
+    def __repr__(self):
+        return '<Hole([{}], {}, {})>'.format(self.shape, self.position, self.label)
+
+class BrassValve:
+    """
+    A valve: the shape of the deviation tube and its location on the main bore.
+
+    Attributes
+    ----------
+    shape : :py:class:`DesignShape <openwind.design.design_shape.DesignShape>`
+        The shape of the deviation tube.
+    position : :py:class:`DesignParameter \
+    <openwind.design.design_parameter.DesignParameter>`
+        The position of the valve on the main bore.
+    label : string
+        The name of the valve.
+    reconnection_position : :py:class:`DesignParameter \
+    <openwind.design.design_parameter.DesignParameter>`
+        The position on the mainbore at which the reconnection of the deviation pipe
+        is connected.
+    """
+
+    def __init__(self, shape, position, label, reconnection_position):
+        self.shape = shape
+        self.position = position
+        self.label = label
+        if reconnection_position.get_value() <= position.get_value():
+            raise ValueError("The return of the deviation pipe {:e} must be "
+                             "placed afterwards its beginning {:e} on the main "
+                             "bore:".format(reconnection_position, position))
+        self.reconnection_position = reconnection_position
+
+    def __repr__(self):
+        return '<BrassValve([{}], {}, {}, {})>'.format(self.shape, self.position,
+                                                       self.label,
+                                                       self.reconnection_position)
 
 class InstrumentGeometry:
     """
     Parse instrument geometry and fingering into OpenWind classes.
 
-    Create an instrument with the bore shape and holes given by the
-    geometry described in the files.
+    Create an instrument with the bore shape, the holes and the valves given
+    by the geometry described in the files.
 
     Parameters
     ----------
-    files : str or list
-        filename or list of data respecting the file format. Up to three files\
-        can be provided respecting the followind order :
-
-        1. the file (or list) containing the main bore geometry
-        2. the file (or list) with the side holes geometry and location
-        3. the file (or list) indicating the fingering chart
-
+    main_bore : str or list
+        filename or list of data respecting the file format with the
+        main bore geometry
+    holes_or_vales : str or list, optional
+        filename or list of data respecting the file format, with the
+        holes and/or valves geometries. The default is None corresponding to
+        an instrument without hole or valve.
+    fingering_chart : str or list, optional
+        filename or list of data respecting the file format, indicating the
+        fingering chart in accordance with the holes and/or valves. The default
+        is None corresponding to no fingering (everything open)
 
 
     **Structure of the files**
@@ -87,7 +126,7 @@ class InstrumentGeometry:
     the list of already-split lines, for instance:
 
         >>> InstrumentGeometry([[0.0, 2e-3], [0.2, 0.01]])
-        <openwind.InstrumentGeometry at ...>
+        <openwind.InstrumentGeometry(1 main bore parts,...)>
 
     *Main bore geometry file*
 
@@ -210,6 +249,9 @@ class InstrumentGeometry:
     holes: list[ :py:class:`Hole \
     <openwind.technical.instrument_geometry.Hole>` ]
         The description of the tone holes
+    valves: list[ :py:class:`BrassValve\
+    <openwind.technical.instrument_geometry.BrassValve>` ]
+        The description of the valves and the associated deviation pipe
     fingering_chart: :py:class:`FingeringChart \
     <openwind.technical.fingering_chart.FingeringChart>`
         The fingerings of the various notes.
@@ -220,28 +262,26 @@ class InstrumentGeometry:
 
     """
 
-    def __init__(self, *files):
+    def __init__(self, main_bore, holes_valves=None, fingering_chart=None):
         self.optim_params = OptimizationParameters()
-        self._create_main_bore_shapes(files[0])
-
+        self._create_main_bore_shapes(main_bore)
         self.holes = list()
-        if len(files) >= 2:
-            self._create_holes_shapes(files[1])
+        self.valves = list()
+        if not holes_valves is None:
+            self._create_holes_shapes(holes_valves)
+        self.fingering_chart = self._create_fingering_chart(fingering_chart)
 
-        if len(files) >= 3:
-            self.fingering_chart = self._create_fingering_chart(files[2])
-        else:
-            self.fingering_chart = self._create_fingering_chart(None)
 
     def __repr__(self):
         return '<openwind.InstrumentGeometry({} main bore parts, {} holes, ' \
-               '{})>'.format(len(self.main_bore_shapes),
-                             len(self.holes), repr(self.fingering_chart))
+               '{} valves, {})>'.format(len(self.main_bore_shapes),
+                                        len(self.holes), len(self.valves),
+                                        repr(self.fingering_chart))
 
     def __str__(self):
-        return(('-'*30 + '\nMain Bore:\n{} \n' + '-'*30 + '\nHoles:\n{} \n' +
+        return(('-'*30 + '\nMain Bore:\n{} \n' + '-'*30 + '\nSide Components:\n{} \n' +
                 '-'*30 + '\nFingering Chart:\n{}\n')
-               .format(self.print_main_bore_shape(), self.print_holes(),
+               .format(self.print_main_bore_shape(), self.print_side_components(),
                        self.fingering_chart))
 
     # %% creation of lists of shape and fingering chart
@@ -324,6 +364,23 @@ class InstrumentGeometry:
             raise ValueError(str(err) + msg).with_traceback(err.__traceback__)
 
 
+    def set_fingering_chart(self, data):
+        """
+        Set the :py:attr:`fingering_chart<openwind.technical.InstrumentGeometry.fingering_chart>`
+
+        Parameters
+        ----------
+        data : list or string or :py:class:`FingeringChart \
+        <openwind.technical.fingering_chart.FingeringChart>` or None
+            `data` is either direct data, or a filename containing the data.
+            If data is None, the default fingering with all holes open
+            is assumed
+        """
+        if type(data) == FingeringChart:
+            self.fingering_chart = data
+        else:
+            self.fingering_chart = self._create_fingering_chart(data)
+
 
     def _create_fingering_chart(self, data):
         """
@@ -345,6 +402,8 @@ class InstrumentGeometry:
             fingerings.
 
         """
+        labels_side_components = self.get_hole_labels() + self.get_valve_labels()
+
         if not data:
             return FingeringChart()
         data = InstrumentGeometry._interpret_data(data)
@@ -353,33 +412,32 @@ class InstrumentGeometry:
         column_titles, remaining_lines = data[0], data[1:]
         assert column_titles[0] == 'label'
         note_names = column_titles[1:]
-        number_of_holes = len(remaining_lines)
-        chart = -np.ones((number_of_holes, len(note_names))) # Initialize at -1
-        hole_labels = []
-        for hole_i, line in enumerate(remaining_lines):
-            hole_label = line[0]
-            if hole_label not in self.get_hole_labels():
-                raise ValueError("Hole '{}' was not defined in holes file. Chose between: {}"
-                                 .format(hole_label, self.get_hole_labels()))
-            hole_labels.append(hole_label)
+        number_side_comp = len(remaining_lines)
+        chart = -np.ones((number_side_comp, len(note_names))) # Initialize at -1
+        chart_comp_labels = []
+        for comp_i, line in enumerate(remaining_lines):
+            comp_label = line[0]
+            if comp_label not in labels_side_components + ['bell']:
+                raise ValueError("Side component '{}' was not defined in holes file. Chose between: {}"
+                                 .format(comp_label, labels_side_components + ['bell']))
+            chart_comp_labels.append(comp_label)
 
             for note_j, s in enumerate(line[1:]):
                 factor = InstrumentGeometry._parse_opening_factor(s)
-                chart[hole_i, note_j] = factor
+                chart[comp_i, note_j] = factor
 
         # Check if fingering chart is valid
         if np.any(chart == -1):
             raise ValueError("Invalid fingering chart: missing data.")
-        # Check if all holes from holes file
-        # have been defined in fingering chart
-        missing = [hole for hole in self.get_hole_labels()
-                        if hole not in hole_labels]
+        # Check if all side components from holes file have been defined in fingering chart
+        missing = [comp for comp in labels_side_components
+                   if comp not in chart_comp_labels]
         if missing:
-            warnings.warn(('Holes {} missing from fingering chart.\n'
+            warnings.warn(('Side components {} missing from fingering chart.\n'
                            'They will be assumed to remain open.').format(missing))
 
-        return FingeringChart(note_names, hole_labels, chart,
-                              other_holes=missing)
+        return FingeringChart(note_names, chart_comp_labels, chart,
+                              other_side_comp=missing)
 
     # %% Reading external files
 
@@ -430,6 +488,11 @@ class InstrumentGeometry:
         """
         with open(filename) as file:
             lines = file.readlines()
+        raw_parts =  InstrumentGeometry._parse_lines(lines)
+        return raw_parts
+
+    @staticmethod
+    def _parse_lines(lines):
         raw_parts = []
         for line in lines:
             contents = InstrumentGeometry._parse_line(line)
@@ -471,10 +534,15 @@ class InstrumentGeometry:
             - closed if 'x' or 'closed'
             - semi-closed if '0.5' or '.5'
 
+        Each valve is:
+        - 'x' "depressed" or "press down"
+        - 'o' "raised" or "open"
+        - '0.5' semi-pressed (why not!)
+
         Parameters
         ----------
         s : string
-            string containing the information about the opening of the hole
+            string containing the information about the opening
 
         Returns
         -------
@@ -948,10 +1016,10 @@ class InstrumentGeometry:
             label = "hole{}".format(i+1)
             if len(raw_hole)>3:
                 position, chimney, radius, type_ = raw_hole
-                self._add_hole(position, chimney, radius, type_, label)
+                self._add_side_component(position, chimney, radius, type_, label)
             else:
                 position, chimney, radius = raw_hole
-                self._add_hole(position, chimney, radius, label=label)
+                self._add_side_component(position, chimney, radius, label=label)
 
     def _parse_holes_new_format(self, raw_holes):
         """
@@ -961,7 +1029,7 @@ class InstrumentGeometry:
             - 'position' or 'x' : the position of the hole on the main pipe (NEEDED)
             - 'radius' or 'r' : the radius of the hole (NEEDED)
             - 'chimney' or 'l' : the chimney height (NEEDED)
-            - 'label' : the label of the hole (optional)
+            - 'label' : the label of the hole (NEEDED)
             - 'type' : the shape type of the chimney tube (optional), only
             'linear' is accepted.
 
@@ -973,21 +1041,71 @@ class InstrumentGeometry:
 
         """
         unifiate_column_name = {'x': 'position', 'position': 'position',
+                                'location': 'position',
                                 'r': 'radius', 'radius': 'radius',
-                                'l': 'chimney', 'chimney':'chimney',
-                                'type':'type_', 'label':'label'}
+                                'l': 'length', 'chimney':'length',
+                                'length':'length',
+                                'type':'type_', 'label':'label',
+                                'variety':'variety', 'reconnection':'reconnection'}
         column_names = list()
         for col in raw_holes[0]:
             column_names += [unifiate_column_name.get(col)]
-        mandatory_columns = ['position', 'chimney', 'radius']
+
+
+        mandatory_columns = ['label', 'position', 'length', 'radius']
         if not all(col in column_names for col in mandatory_columns):
-            raise ValueError("Hole file must contain columns 'position',"
-                             " 'chimney', 'radius'. See documentation of "
-                             "openwind hole files.")
+            raise ValueError("Hole file must contain columns {}."
+                             "See documentation of openwind hole "
+                             "files.".format(mandatory_columns))
 
         for i, line in enumerate(raw_holes[1:]):
             hole_data = dict(zip(column_names, line))
-            self._add_hole(**hole_data)
+            self._add_side_component(**hole_data)
+
+    def _add_side_component(self, position, length, radius, type_='linear',
+                          label=None, variety='hole', reconnection=None):
+        if label is None:
+            raise ValueError("Side component (hole or valve) needs a label")
+        if label in self.get_hole_labels() + self.get_valve_labels():
+            raise ValueError("Several side components (hole or valve) were defined with the same label.")
+
+        if variety.lower() == 'hole':
+            self._add_hole(position=position, chimney=length, radius=radius,
+                          type_=type_, label=label)
+        elif variety.lower() == 'valve':
+            self._add_valve(position=position, length=length, radius=radius,
+                            reconnection=reconnection, type_=type_, label=label,)
+        else:
+            raise ValueError('Unknown side component variety, chose between'
+                             ' "hole" and "valve"')
+
+    def _add_valve(self, position, length, radius, reconnection=None, type_='linear',
+                   label=None):
+        if reconnection is None:
+            raise ValueError('Valve needs its reconnection location on the main bore'
+                             'in column "reconnection"')
+
+        main_bore_entry = self.__localize_hole(position)
+        entry_position = self.__hole_position_designparameter(position,
+                                                              main_bore_entry,
+                                                              label)
+
+        main_bore_reconnection = self.__localize_hole(reconnection)
+        reconnection_position = self.__hole_position_designparameter(reconnection,
+                                                                main_bore_reconnection,
+                                                                label + '_reconnection')
+
+        shape_X = [self.__designparameter(0.0, label + '_dev_pipe_entry')]
+        shape_X.append(self.__designparameter(length, label + '_length'))
+
+        r_entry = self.__hole_radius_designparameter(radius, main_bore_entry,
+                                                     entry_position, label)
+
+        r_reconnection = self.__hole_radius_designparameter(radius, main_bore_reconnection,
+                                                       reconnection_position, label + '_reconnection')
+
+        new_shape = self._build_shape(shape_X, [r_entry, r_reconnection], 'linear', [])
+        self.valves.append(BrassValve(new_shape, entry_position, label, reconnection_position))
 
 
     def _add_hole(self, position, chimney, radius, type_='linear', label=None):
@@ -1017,7 +1135,7 @@ class InstrumentGeometry:
             list of the Hole object.
 
         """
-        if type_ != 'linear' and type_ != 'cone' and type_ != 'cylinder':
+        if type_ not in ['linear', 'cone', 'cylinder']:
             raise ValueError("Only cylindrical holes are implemented (type='linear')")
         if label is None:
             raise ValueError("Hole needs a label")
@@ -1047,27 +1165,39 @@ class InstrumentGeometry:
         """
         return [hole.label for hole in self.holes]
 
-    def get_bore_list(self):
+    def get_valve_labels(self):
+        """
+        Returns
+        -------
+        list
+            The labels of the brass valves.
+        """
+        return [valve.label for valve in self.valves]
+
+
+    def get_bore_list(self, all_fields=False):
         """
         Return the list of data of the geometry.
 
+        Parameters
+        ----------
+        all_fields: bool, optional
+            Indicate all the fields for the side components even if they are not
+            needed. Default: False
+
         Returns
         -------
         list
             The list of the data of the main bore.
         list
-            The list of data for the holes.
+            The list of data for the side components.
 
         """
         bore_list = []
         for shape in self.main_bore_shapes:
             bore_list.append(self._parse_line(str(shape)))
-        holes_list = []
-        for hole in self.holes:
-            _, chimney = hole.shape.get_endpoints_position()
-            _, radius = hole.shape.get_endpoints_radius()
-            holes_list.append([str(hole.label), str(hole.position), str(chimney), str(radius)])
-        return bore_list, holes_list
+        side_list = self._parse_lines(self.print_side_components(all_fields=all_fields).split('\n'))
+        return bore_list, side_list
 
     def get_main_bore_length(self):
         """
@@ -1076,11 +1206,11 @@ class InstrumentGeometry:
         Returns
         -------
         float
-            Total length of the main bore (or position of the last end of the
-            main bore).
+            Total length of the main bore
 
         """
-        return self.main_bore_shapes[-1].get_position_from_xnorm(1)
+        mb_length = [shape.get_length() for shape in self.main_bore_shapes]
+        return sum(mb_length)
 
     def get_main_bore_radius_at(self, position):
         """
@@ -1105,8 +1235,184 @@ class InstrumentGeometry:
                 radius[is_in] = shape.get_radius_at(x_norm[is_in])
         return radius
 
+    # %% Modify the geometry
+
+    def shift_x_axis(self, offset):
+        """
+        Shift the x-axis by the specified offset.
+
+        It shifts the value of all the design parameters related to the x-axis:
+
+        - the bounds of the main bore shapes
+        - the positions of the spline nodes
+        - the holes' positions
+
+        Parameters
+        ----------
+        offset : float
+            The offset in meter (positive or negative).
+
+        """
+
+        shifted = list()
+
+        bounds_tuple = [shape.get_endpoints_position() for shape in self.main_bore_shapes]
+        spline_tuple = [shape.X for shape in self.main_bore_shapes if type(shape) is Spline]
+        mb_pos = [pos for bound in bounds_tuple + spline_tuple for pos in bound]
+        hole_pos = [hole.position for hole in self.holes]
+        valve_pos = [valve.position for valve in self.valves]
+        valve_reco = [valve.reconnection_position for valve in self.valves]
+
+        position = hole_pos + mb_pos + valve_pos + valve_reco
+
+        for param in position:
+            if param not in shifted:
+                if type(param) is FixedParameter:
+                    param._value += offset
+                else:
+                    self.optim_params.values[param.index] += offset
+                    self.optim_params.bounds[param.index] = tuple([b + offset for b in self.optim_params.bounds[param.index]])
+                shifted.append(param)
+
+
+    def __add__(self, other):
+
+        """
+        Concatenate two InstrumentGeometry.
+
+        The x-axis of the second :py:class:`InstrumentGeometry<openwind.technical.instrument_geometry.InstrumentGeometry>`
+        is shifted such as its begining correspond to the end of the first one.
+
+        .. warning::
+            The addition of InstrumentGeometry does not manage yet the
+            Fingering chart!
+
+        To avoid problem with the gestion of the parameters etc, the simpliest
+        is to generate the list corresponding ot the new geomtry and instantiate
+        a totally new InstrumentGeometry from it.
+
+        Parameters
+        -----------
+        other : :py:class:`InstrumentGeometry<openwind.technical.instrument_geometry.InstrumentGeometry>`
+            The second instrument wich must be added dowstream this one
+
+        Returns
+        --------
+        :py:class:`InstrumentGeometry<openwind.technical.instrument_geometry.InstrumentGeometry>`
+            The concatenation of the two instruments
+
+        """
+        # get the list of the actual geometry
+        mb_self, side_self = self.get_bore_list(all_fields=True)
+        n_side_self = len(self.holes) + len(self.valves)
+
+        if type(other) is InstrumentGeometry:
+            # shift the other geometry such its begin start at the end of self
+            starting_other = other.main_bore_shapes[0].get_position_from_xnorm(0)
+            offset = self.main_bore_shapes[-1].get_position_from_xnorm(1) - starting_other
+            other.shift_x_axis(offset)
+
+            # get the list of the other-shifted instrument
+            mb_other, side_other = other.get_bore_list(all_fields=True)
+            n_side_other = len(other.holes) + len(other.valves)
+            # remove the second head-line
+            side_other = side_other[1:]
+
+            # restore the initial offset of the other instrument
+            other.shift_x_axis(-offset)
+
+            if len(self.fingering_chart.all_notes() + other.fingering_chart.all_notes()) != 0:
+                warnings.warn('The addition of InstrumentGeometry does not manage '
+                              'yet the Fingering chart!')
+        elif other == 0: # the addition with 0 resinstanciate the same instrument (necessary to use "sum")
+            mb_other = list()
+            side_other = list()
+        else:
+            raise TypeError("can only concatenate 'InstrumentGeometry' (not '{}') to 'InstrumentGeometry'".format(type(other).__name__))
+
+        # combine and instanciate a new instrument
+        return InstrumentGeometry(mb_self + mb_other, side_self + side_other)
+
+    def __radd__(self, other):
+        if other == 0:
+            return self.__add__(other)
+        else:
+            return other.__add__(self)
+
+    def extract(self, start, stop=np.inf):
+        """
+        Extract a part of this instrument between two positions
+
+        Get a new InstrumenteGeometry corresponding to this one, cut between
+        the two indicated  position.
+
+        .. warning::
+            The excratction of InstrumentGeometry does not manage yet the
+            Fingering chart!
+
+        Parameters
+        -----------
+        start: float
+            The position (in meter) of the "left" end of the slice.
+            (You can indicate -np.Inf to keep the original end)
+        stop: float
+            The position (in meter) of the "rigth" end of the slice.
+            (You can indicate +np.Inf to keep the original end)
+
+        Returns
+        --------
+        :py:class:`InstrumentGeometry<openwind.technical.instrument_geometry.InstrumentGeometry>`
+            The extracted instrument.
+
+        """
+        mb_self, side_self = self.get_bore_list(all_fields=True)
+        x0_mb = [shape.get_position_from_xnorm(0) for shape
+                 in self.main_bore_shapes]
+        x1_mb = [shape.get_position_from_xnorm(1) for shape
+                 in self.main_bore_shapes]
+        # get the shape which must be included in the new instrument
+        extract_mb = [part for k, part in enumerate(mb_self) if x0_mb[k]<stop
+                    and x1_mb[k]>start]
+
+        # get the holes which must be included in the new isntrument
+        if len(side_self)>0:
+            x_hole = [hole.position.get_value() for hole in self.holes]
+            x_valve = [valve.position.get_value() for valve in self.valves]
+            x_reco =  [valve.reconnection_position.get_value() for valve in self.valves]
+            if (any(np.logical_and(np.array(x_valve)<stop, np.array(x_reco)>stop))
+                or any(np.logical_and(np.array(x_valve)<start, np.array(x_reco)>start))):
+                raise ValueError('It is impossible to slice an instrument in between the 2 extremities of a valve.')
+            x_pos = sorted(x_hole + x_valve)
+            extract_side = [side_self[0]]
+
+            extract_side += [side for k, side in enumerate(side_self[1:])
+                           if x_pos[k]<stop and x_pos[k]>=start]
+        else:
+            extract_side = list()
+
+        if len(self.fingering_chart.all_notes()) != 0:
+            warnings.warn('The extracting of InstrumentGeometry does not manage '
+                          'yet the Fingering chart!')
+        # create a new instrument from the lists
+        extract_geom = InstrumentGeometry(extract_mb, extract_side)
+
+        # cut the first shape if needed
+        geom_entrance = extract_geom.main_bore_shapes[0]
+        if geom_entrance.get_position_from_xnorm(0) < start:
+            x_stop = geom_entrance.get_position_from_xnorm(1)
+            geom_entrance.cut_shape(start, x_stop)
+
+        # cut the last shape if needed
+        geom_end = extract_geom.main_bore_shapes[-1]
+        if geom_end.get_position_from_xnorm(1) > stop:
+            x_start = geom_end.get_position_from_xnorm(0)
+            geom_end.cut_shape(x_start, stop)
+
+        return extract_geom
+
+
     # %% Plot
-    def plot_InstrumentGeometry(self, figure=None, note=None, double_plot=True, **kwargs):
+    def plot_InstrumentGeometry(self, figure=None, note=None, double_plot=True, label='_', **kwargs):
         """
         Display the geometry and holes of the instrument.
 
@@ -1124,9 +1430,6 @@ class InstrumentGeometry:
             Additional arguments are passed to the `plt.plot` function.
 
         """
-        if self.holes == []:
-            double_plot = False
-        # double_plot = (self.holes != [])
         mmeter = 1e3
 
         if not figure:
@@ -1134,26 +1437,52 @@ class InstrumentGeometry:
         else:
             fig = figure
         ax = fig.get_axes()
+
+        if self.holes == []  and len(ax) < 2:
+            double_plot = False
+
         if len(ax) < 2 and double_plot:
             ax = [fig.add_subplot(2, 1, 1)]
             ax.append(fig.add_subplot(2, 1, 2, sharex=ax[0]))
         elif len(ax) < 1:
             ax.append(fig.add_subplot(1, 1, 1))
 
-        x = np.linspace(0, 1, 1000)
-        for shape in self.main_bore_shapes:
-            radius = np.append(shape.get_radius_at(x), np.nan)
-            position = np.append(shape.get_position_from_xnorm(x), np.nan)
-            ax[0].plot(np.append(position, np.flip(position))*mmeter,
-                     np.append(radius, np.flip(-radius))*mmeter, **kwargs)
-            if double_plot:
-                ax[1].plot(np.append(position, np.flip(position))*mmeter,
-                         np.append(radius, np.flip(-radius))*mmeter, **kwargs)
+        self._plot_shape(self.main_bore_shapes[0], ax, mmeter, shift_x=0,
+                         shift_y=0, label=label, **kwargs)
+        for shape in self.main_bore_shapes[1:]:
+            self._plot_shape(shape, ax, mmeter, shift_x=0, shift_y=0, **kwargs)
 
+        self._plot_holes(ax, mmeter, note, **kwargs)
+
+        self._plot_valves(ax, mmeter, **kwargs)
+
+        if label != '_':
+            ax[0].legend()
+
+        ax[-1].set_xlabel('Position (mm)')
+        ax[0].set_ylabel('Radius (mm)')
+        ax[0].axis('equal')
+        if double_plot:
+            ax[1].axis('equal')
+            ax[1].set_ylabel('Radius (mm)')
+
+    def _plot_shape(self, shape, ax, mmeter, shift_x=0, shift_y=0, **kwargs):
+        x = np.linspace(0, 1, 1000)
+        radius = np.append(shape.get_radius_at(x), np.nan)
+        position = np.append(shape.get_position_from_xnorm(x), np.nan) + shift_x
+        line= ax[0].plot(np.append(position, np.flip(position))*mmeter,
+                         (np.append(radius, np.flip(-radius)) + shift_y)*mmeter,
+                         **kwargs)
+        if len(ax)>1:
+            ax[1].plot(np.append(position, np.flip(position))*mmeter,
+                       (np.append(radius, np.flip(-radius)) + shift_y)*mmeter, **kwargs)
+        return line
+
+    def _plot_holes(self, ax, mmeter, note, **kwargs):
         if note:
             fingering = self.fingering_chart.fingering_of(note)
             def plot_or_fill(axes, hole):
-                if fingering.is_hole_open(hole.label):
+                if fingering.is_side_comp_open(hole.label):
                     return axes.plot
                 else:
                     return axes.fill
@@ -1161,6 +1490,7 @@ class InstrumentGeometry:
             def plot_or_fill(axes, hole):
                 return axes.plot
 
+        x = np.linspace(0, 1, 10)
         for hole in self.holes:
             position = hole.position.get_value()
             radius = hole.shape.get_radius_at(x)
@@ -1174,18 +1504,40 @@ class InstrumentGeometry:
             if type(hole_plot[0]).__name__ == 'Polygon':
                 hole_plot[0].set_edgecolor(hole_plot[0].get_facecolor())
 
-            if double_plot:
+            if len(ax)>1:
                 theta = np.linspace(0,2*np.pi,100)
                 hole_plot = plot_or_fill(ax[1], hole)((position + np.mean(radius)*np.cos(theta))*mmeter,
                                                       np.mean(radius)*np.sin(theta)*mmeter, **kwargs)
                 if type(hole_plot[0]).__name__ == 'Polygon':
                     hole_plot[0].set_edgecolor(hole_plot[0].get_facecolor())
-        ax[-1].set_xlabel('Position (mm)')
-        ax[0].set_ylabel('Radius (mm)')
-        ax[0].axis('equal')
-        if double_plot:
-            ax[1].axis('equal')
-            ax[1].set_ylabel('Radius (mm)')
+
+    def _plot_valves(self, ax, mmeter, **kwargs):
+        for valve in self.valves:
+            pos = valve.position.get_value()
+            reco_pos = valve.reconnection_position.get_value()
+            length = valve.shape.get_length()
+
+            shift_x = (pos + reco_pos - length)*.5
+
+            rad = max(valve.shape.get_radius_at(np.linspace(0,1,10)))
+            shift_y = ax[0].get_ylim()[0]/mmeter - rad -5e-3
+            line = self._plot_shape(valve.shape, ax, mmeter, shift_x, shift_y,
+                                    **kwargs)
+
+            main_bore = self.__localize_hole(pos)
+            pos_norm = main_bore.get_xnorm_from_position(pos)
+            main_radius = main_bore.get_radius_at(pos_norm)
+
+            main_bore_reco = self.__localize_hole(reco_pos)
+            pos_norm_reco = main_bore_reco.get_xnorm_from_position(reco_pos)
+            main_radius_reco = main_bore_reco.get_radius_at(pos_norm_reco)
+
+            for axe in ax:
+                axe.plot(np.array([pos, pos, shift_x, np.nan, shift_x+length, reco_pos, reco_pos])*mmeter,
+                         np.array([main_radius, -main_radius, shift_y+rad, np.nan,
+                                   shift_y+rad, -main_radius_reco, main_radius_reco])*mmeter,
+                         ':', color=line[0].get_color())
+
 
     # %% print files
     def print_files(self, generic_name, extension='.txt'):
@@ -1221,8 +1573,8 @@ class InstrumentGeometry:
             generic_name = generic_name[:-4]
             extension = '.csv'
         self.print_main_bore_shape(generic_name + '_MainBore' + extension)
-        if self.holes:
-            self.print_holes(generic_name + '_Holes' + extension)
+        if self.holes or self.valves:
+            self.print_side_components(generic_name + '_SideComponents' + extension)
         if len(self.fingering_chart.all_notes()) > 0:
             filename = generic_name + '_FingeringChart' + extension
             f = open(filename, "w")
@@ -1255,6 +1607,9 @@ class InstrumentGeometry:
             f.write(msg)
 
     def print_holes(self, filename=None):
+        return self.print_side_components(filename)
+
+    def print_side_components(self, filename=None, all_fields=False):
         """
         Print the holes information.
 
@@ -1263,6 +1618,9 @@ class InstrumentGeometry:
         filename : string, optional
             If indicated, the filename in which the holes informatiions are
             written. The default is None.
+        all_fields: bool, optional
+            If true, print all the fields (even variety and recoonection,
+            if there is no valve). Necessary for the addition of several instrument.
 
         Returns
         -------
@@ -1270,16 +1628,28 @@ class InstrumentGeometry:
             The string corresponding to the holes information.
 
         """
-        if self.holes:
-            col_names = ['label', 'position', 'chimney', 'radius']
-            rows = []
-            for hole in self.holes:
-                _, chimney = hole.shape.get_endpoints_position()
-                _, radius = hole.shape.get_endpoints_radius()
-                rows.append([hole.label, hole.position, chimney, radius])
+
+        col_names = ['label', 'variety',  'position', 'length', 'radius', 'reconnection']
+        rows = list()
+        for hole in self.holes:
+            length = hole.shape.get_endpoints_position()[1]
+            radius = hole.shape.get_endpoints_radius()[1]
+            rows.append([hole.label, 'hole', hole.position, length, radius, '/'])
+        for valve in self.valves:
+            length = valve.shape.get_endpoints_position()[1]
+            radius = valve.shape.get_endpoints_radius()[0]
+            rows.append([valve.label, 'valve', valve.position, length, radius, valve.reconnection_position])
+        # sort wr to the position of the side components
+        rows.sort(key=lambda i: i[2].get_value())
+        if self.valves or all_fields:
             msg = tabulate(rows, col_names)
+        elif self.holes:
+            rows_simple = [[row[0]] +row[2:5] for row in rows]
+            msg = tabulate(rows_simple, [col_names[0]] + col_names[2:5])
         else:
-            msg = 'None'
+            msg = '' #'None'
+
+
         if not filename:
             return msg
         else:
