@@ -88,7 +88,11 @@ class Player:
     def __init__(self, dict_key='UNITARY_FLOW', note_events=[],
                  transition_duration=.02):
         if type(dict_key) == str:
-            dict_to_load = getattr(EXCITATOR_DEFAULTS, dict_key)
+            if not dict_key in AVAILABLE_DEFAULTS:
+                raise AttributeError('Unknown default excitator. Chose between'
+                                     ' {}'.format(AVAILABLE_DEFAULTS))
+            else:
+                dict_to_load = getattr(EXCITATOR_DEFAULTS, dict_key)
         else:
             dict_to_load = dict_key
 
@@ -252,7 +256,6 @@ class Player:
         for k, label in enumerate(curves):
             ax.append(fig.add_subplot(n_row, 1, k+2, sharex=ax[0]))
             self.plot_one_control(label, time, ax[k+1])
-        plt.show()
         fig.suptitle(title)
 
     def __repr__(self):
