@@ -76,6 +76,68 @@ from numpy import pi
 #         not yet implemented
 
 
+# %%  FUNCTION OPERATIONS
+
+
+def add(f, g):
+    """
+    Adding two functions (temporal curves) together
+
+    Parameters
+    ----------
+    f : function
+    g : function
+
+    Returns
+    -------
+    function
+        return new function h = f + g
+
+    """
+    def env(t):
+        return f(t) + g(t)
+    return env
+
+
+def substract(f, g):
+    """
+    Substracting a function (temporal curve) from another
+
+    Parameters
+    ----------
+    f : function
+    g : function
+
+    Returns
+    -------
+    function
+        return new function h = f - g
+
+    """
+    def env(t):
+        return f(t) - g(t)
+    return env
+
+
+def multiply(f, g):
+    """
+    Multiplying two functions (temporal curves) together
+
+    Parameters
+    ----------
+    f : function
+    g : function
+
+    Returns
+    -------
+    function
+        return new function h = f * g
+
+    """
+    def env(t):
+        return f(t) * g(t)
+    return env
+
 # %%  COMPOSITE FUNCTIONS (PROBABLY MOST USED)  %% #
 
 
@@ -142,7 +204,7 @@ def ADSR(t1, t2, amplitude,
     """
     Attack-Decay-Sustain-Release envelope
 
-    .. image:: /images/ADSR.png
+    .. image :: https://files.inria.fr/openwind/pictures/ADSR.png
 
     Parameters
     ----------
