@@ -29,7 +29,7 @@ from openwind.continuous import InstrumentPhysics
 from openwind.temporal import (TemporalSolver, RecordingDevice)
 
 
-def simulate(duration, *files,
+def simulate(duration, main_bore, holes_valves=None, fingering_chart=None,
              player = None,
              l_ele=None, order=None,
              nondim=False,
@@ -51,9 +51,21 @@ def simulate(duration, *files,
     ----------
     duration : float
         Duration of the simulation, in (simulated-world) seconds.
-    *files : file_bore[, file_holes[, file_fingering_chart]]
-        Description of the instrument.
-        See also : :py:class:`InstrumentGeometry <openwind.technical.instrument_geometry.InstrumentGeometry>`
+    main_bore : str or list
+        filename or list of data respecting the file format with the
+        main bore geometry. See also : :py:class:`InstrumentGeometry \
+        <openwind.technical.instrument_geometry.InstrumentGeometry>`
+    holes_or_vales : str or list, optional
+        filename or list of data respecting the file format, with the
+        holes and/or valves geometries. The default is None corresponding to
+        an instrument without hole or valve. See also : :py:class:`InstrumentGeometry \
+        <openwind.technical.instrument_geometry.InstrumentGeometry>`
+    fingering_chart : str or list, optional
+        filename or list of data respecting the file format, indicating the
+        fingering chart in accordance with the holes and/or valves. The default
+        is None corresponding to no fingering (everything open).
+        See also : :py:class:`InstrumentGeometry \
+        <openwind.technical.instrument_geometry.InstrumentGeometry>`
     player : openwind.Player
         Description of the musician's control (embouchure and fingerings)
     l_ele, order :
@@ -88,7 +100,7 @@ def simulate(duration, *files,
         warnings.warn('The default temperature is 25 degrees Celsius.')
 
 
-    instrument_geometry = InstrumentGeometry(*files)
+    instrument_geometry = InstrumentGeometry(main_bore, holes_valves, fingering_chart)
     instru_physics = InstrumentPhysics(instrument_geometry, temperature,
                                        player, losses=losses,
                                        radiation_category=radiation_category,
