@@ -80,6 +80,11 @@ class SimulationTracker:
         self._t0 = time.perf_counter()
         self._next_estimate = self._t0 + self._delay_before_first_estimate
 
+        # Display a welcome message as we start
+        if self._is_display_enabled:
+            self._display_welcome()
+
+
     def enable_display(self, enable):
         """
         Modify whether to display the info automatically
@@ -93,10 +98,6 @@ class SimulationTracker:
 
     def update(self):
         """Track the simulation at each step, and display if necessary."""
-        # Display a welcome message at the first step
-        if self._steps_done == 0 and self._is_display_enabled:
-            self._display_welcome()
-
         self._steps_done += 1
         if self._should_display():
             self.display()
